
	/*******************************************************************
	** Some routines which deal with files (reading, writing, etc.)
	*******************************************************************/

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <jpeglib.h>
#include "globals.h"


	/*******************************************************************
	** Read a ppm/pgm or jpeg image from a file.  Returns the number
	** of bytes per pixel after a successful read, 0 otherwise.
	*******************************************************************/

int ReadImage(char			*filename,			/* complete name of image to read */
			  unsigned char	**raw,				/* image data, returned */
			  int			*ROWS,int *COLS)	/* size of image, returned */

{
FILE							*fpt;
int								i,r,BYTES,BPP;
char							text[30];
struct jpeg_decompress_struct	cinfo;
struct jpeg_error_mgr			jerr;
JSAMPARRAY						buffer;
HDC								hDC;
char							LoadText[250];


strcpy(file_comments,"");
if ((fpt=fopen(filename,"rb")) == NULL)
  {	  // let caller display warning message; here just return 0 
  // MessageBox(NULL,filename,"Unable to open file:",MB_APPLMODAL | MB_OK);
  return(0);
  }

		/* see if in jpeg format */
text[0]=getc(fpt); text[1]=getc(fpt);
if (text[0] == -1  &&  text[1] == -40)
  {		/* is in jpeg format -- start over and read in */
  fclose(fpt);
  fpt=fopen(filename,"rb");
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  jpeg_stdio_src(&cinfo, fpt);
  jpeg_read_header(&cinfo, TRUE);
  jpeg_start_decompress(&cinfo);
  *COLS=cinfo.output_width;
  *ROWS=cinfo.output_height;
  BPP=cinfo.output_components;
  if ((*raw=(unsigned char *)calloc((*ROWS)*(*COLS)*BPP,1)) == NULL)
    {
    MessageBox(MainWnd,"Unable to allocate memory","ReadImage()",MB_APPLMODAL | MB_OK);
	return(0);
    }
  r=0;
  buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo, JPOOL_IMAGE, (*COLS)*BPP, 1);
  hDC=GetDC(MainWnd);
  sprintf(LoadText,"Loading %s ...",filename);
  while (r < *ROWS)
    {
	if (r%50 == 0)
	  {
	  strcat(LoadText,".");
	  TextOut(hDC,0,0,LoadText,strlen(LoadText));
	  }
    jpeg_read_scanlines(&cinfo, buffer, 1);
    for (i=0; i<(*COLS)*BPP; i++)
      (*raw)[r*(*COLS)*BPP+i]=(unsigned char)buffer[0][i];
    r++;
    }
  ReleaseDC(MainWnd,hDC);
  jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);
  fclose(fpt);
  return(BPP);
  }
fclose(fpt);
fpt=fopen(filename,"rb");	/* try again, in ppm format */
text[0]=getc(fpt); text[1]=getc(fpt); text[2]=getc(fpt); text[2]='\0';
if (strcmp(text,"P5") == 0  ||  strcmp(text,"P6") == 0)
  {
  if (strcmp(text,"P5") == 0)
    BPP=1;
  else
    BPP=3;
  (*ROWS)=(*COLS)=BYTES=-1;
  i=0;
  while (BYTES == -1)
    {
    fscanf(fpt,"%s",text);
    if (text[0] == '#') /* comment -- ignore */
      {
	  strcat(file_comments,text);
	  i=strlen(file_comments);
      while ((text[0]=fgetc(fpt)) != '\n')
		{ file_comments[i]=text[0]; i++; }
	  file_comments[i]='\0';
      continue;
      }
    if ((*COLS) == -1)
      (*COLS)=atoi(text);
    else if ((*ROWS) == -1)
      (*ROWS)=atoi(text);
    else
      BYTES=atoi(text);
    }  
  if (BYTES != 255)
    {
    MessageBox(MainWnd,filename,"File is not 8-bit ppm format",MB_APPLMODAL | MB_OK);
	return(0);
    }
  if (((*raw)=(unsigned char *)calloc((*ROWS)*(*COLS)*BPP,1)) == NULL)
	{
    MessageBox(MainWnd,"Unable to allocate memory (raw)","ReadImage()",MB_APPLMODAL | MB_OK);
    sprintf(text,"%d x %d x %d bytes?\n",*COLS,*ROWS,BPP);
    MessageBox(MainWnd,text,"ReadImage()",MB_APPLMODAL | MB_OK);
	return(0);
    }
  text[0]=getc(fpt);
  fread(*raw,1,BPP*(*ROWS)*(*COLS),fpt);
  fclose(fpt);
  return(BPP);
  }
fclose(fpt);
MessageBox(MainWnd,filename,"Unknown image format",MB_APPLMODAL | MB_OK);
return(0);
}



	/*******************************************************************
	** Saves an image to a file in either jpeg (FileType=1) or PPM/PGM
	** (FileType=2) format.  Returns a 1 after a successful save, 0 otherwise.
	*******************************************************************/

int WriteImage(char			 *filename,		/* complete name of image to save */
			   unsigned char *image,		/* image data, RGB L->R T->D format */
			   int			 ROWS,int COLS,	/* size of image */
			   int			 BPP,			/* bytes per pixel of image */
			   int			 FileType)		/* 2 jpeg, 1 PPM/PGM */

{
FILE							*fpt;
struct jpeg_compress_struct		cinfo;
struct jpeg_error_mgr			jerr;
JSAMPROW						row_pointer[1];
HDC								hDC;
char							SaveText[250];

if ((fpt=fopen(filename,"wb")) == NULL)
  {
  MessageBox(MainWnd,filename,"Unable to open for writing:",MB_APPLMODAL | MB_OK);
  return(0);
  }
if (FileType == 2)
  {
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);
  jpeg_stdio_dest(&cinfo, fpt);
  cinfo.image_width = COLS;
  cinfo.image_height = ROWS;
  cinfo.input_components = BPP;
  if (BPP == 1)
	cinfo.in_color_space=JCS_GRAYSCALE;
  else
    cinfo.in_color_space = JCS_RGB;
  jpeg_set_defaults(&cinfo);
  jpeg_set_quality(&cinfo, 100, TRUE );
  jpeg_start_compress(&cinfo, TRUE);
  hDC=GetDC(MainWnd);
  sprintf(SaveText,"Writing %s ...",filename);
  while (cinfo.next_scanline < cinfo.image_height)
    {
    if ((cinfo.next_scanline)%50 == 0)
      {
      strcat(SaveText,".");
      TextOut(hDC,0,0,SaveText,strlen(SaveText));
      }
    row_pointer[0] = & image[cinfo.next_scanline*COLS*BPP];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
    }
  ReleaseDC(MainWnd,hDC);
  jpeg_finish_compress(&cinfo);
  fclose(fpt);
  jpeg_destroy_compress(&cinfo);
  return(1);
  }
else if (FileType == 1)
  {
  fprintf(fpt,"P%d %d %d 255\n",(BPP == 1 ? 5 : 6),COLS,ROWS);
  fwrite(image,1,ROWS*COLS*BPP,fpt);
  fclose(fpt);
  return(1);
  }
else
  {
  MessageBox(NULL,"Unknown File Type","WriteImage() Error",MB_OK | MB_APPLMODAL);
  fclose(fpt);
  return(0);
  }
}
