
/*
**	This file contains routines for computing and working with
**	histograms.
*/

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <windows.h>
#include "resource.h"
#include "globals.h"

/*
**	This routine creates a 256x256 image showing the histogram.
*/

void MakeHistogramImage(unsigned char *InputImage,
					 unsigned char *HistImage,
					 int nrows,int ncols,int nbytes)

{
int		hist[256*3],i,b,w,r,c,largest;

for (i=0; i<256*3; i++)
  hist[i]=0;
for (b=0; b<nbytes; b++)
  for (i=0; i<nrows*ncols; i++)
    hist[256*b+InputImage[nbytes*i+b]]++;

largest=0;
for (i=1; i<256*3; i++)
  if (hist[i] > hist[largest])
	largest=i;

if (nbytes == 1)
  {
  for (i=0; i<266*266; i++)
    HistImage[i]=255;
  for (c=0; c<256; c++)
    for (r=256; r>=0; r--)
	  {
	  HistImage[(r+5)*266+c+5]=0;
	  if (256-r > (int)((double)hist[c]/(double)hist[largest]*255.0))
	    break;
      }
  }
else
  {
  for (i=0; i<266*266*3; i++)
    HistImage[i]=0;
  for (c=0; c<256; c++)
	for (w=0; w<3; w++)
	  {
	  if (w == 0)
	    {
		if (hist[c] > hist[256+c]  &&  hist[c] > hist[512+c]) b=0;
		else if (hist[256+c] > hist[c]  &&  hist[256+c] > hist[512+c]) b=1;
		else b=2;
		}
	  else if (w == 2)
	    {
		if (hist[c] < hist[256+c]  &&  hist[c] < hist[512+c]) b=0;
		else if (hist[256+c] < hist[c]  &&  hist[256+c] < hist[512+c]) b=1;
		else b=2;
		}
	  else
	    {
		if ((hist[c] < hist[256+c]  &&  hist[c] > hist[512+c]) ||
			(hist[c] > hist[256+c]  &&  hist[c] < hist[512+c])) b=0;
		else if ((hist[256+c] < hist[c]  &&  hist[256+c] > hist[512+c]) ||
			(hist[256+c] > hist[c]  &&  hist[256+c] < hist[512+c])) b=1;
		else b=2;
		}
      for (r=256; r>=0; r--)
	    {
	    HistImage[3*((r+5)*266+c+5)+b]=255;
	    if (256-r > (int)((double)hist[c+(b*256)]/(double)hist[largest]*255.0))
	      break;
        }
      }
  }
}


void InvertImage(unsigned char *InputImage,
				 unsigned char *OutputImage,
				 int nrows,int ncols,int bytes)

{
int i;

for (i=0; i<nrows*ncols*bytes; i++)
  OutputImage[i]=255-InputImage[i];
}



	/*
	** Take the avg of three bands, or red, green, or blue band,
	** from an RGB image.
	*/

void GrayImage(unsigned char *InputImage,
				 unsigned char *OutputImage,
				 int nrows,int ncols,
				 int Conversion)  /* 0 => avg, 1 => R, 2 => G, 3 => B */

{
int i;

if (Conversion == 0)
  for (i=0; i<nrows*ncols; i++)
    OutputImage[i]=(unsigned char)((int)(InputImage[i*3+0]+InputImage[i*3+1]+InputImage[i*3+2])/3);
else
  for (i=0; i<nrows*ncols; i++)
    OutputImage[i]=InputImage[i*3+Conversion-1];
}



void RandomColorImage(unsigned char *InputImage,
					  unsigned char *OutputImage,
					  int nrows,int ncols,int bytes)

{
int i,j,map[3][256];

srand((unsigned)time(NULL));
for (i=0; i<bytes; i++)
  for (j=0; j<256; j++)
	map[i][j]=(rand() % 256);
if (bytes == 1)
  {
  for (j=0; j<nrows*ncols; j++)
	{
	OutputImage[j*3+0]=map[0][InputImage[j]];
	OutputImage[j*3+1]=map[1][InputImage[j]];
	OutputImage[j*3+2]=map[2][InputImage[j]];
	}
  }
else
  {
  for (i=0; i<3; i++)
    for (j=0; j<nrows*ncols; j++)
      OutputImage[j*bytes+i]=map[i][InputImage[j*bytes+i]];
  }
}



void PlotImageLine(unsigned char *InputImage,
				   unsigned char *OutputImage,
				   int ROWS,int COLS,int BPP,
				   int *indices,int tp)

{
int		i;

for (i=0; i<256*tp*BPP; i++)
  OutputImage[i]=0;
if (BPP == 1)
  for (i=0; i<tp; i++)
    OutputImage[(255-InputImage[indices[i]])*tp+i]=255;
else
  for (i=0; i<tp; i++)
	{
    OutputImage[((255-InputImage[indices[i]*3+0])*tp+i)*3+0]=255;
    OutputImage[((255-InputImage[indices[i]*3+1])*tp+i)*3+1]=255;
    OutputImage[((255-InputImage[indices[i]*3+2])*tp+i)*3+2]=255;
	}
}



void RegionProperties(unsigned char *InputImage,
					  int ROWS,int COLS,int BPP,
					  int y,int x,
					  int *Area,int *CenterX,int *CenterY,
					  double *Compactness)
{
int	  r,c,i,border,border_pixel;

(*Area)=(*CenterX)=(*CenterY)=border=0;
for (r=0; r<ROWS; r++)
  for (c=0; c<COLS; c++)
	{
	for (i=0; i<BPP; i++)
	  if (InputImage[(r*COLS+c)*BPP+i] != InputImage[(y*COLS+x)*BPP+i])
		break;
	if (i < BPP)
	  continue;
	border_pixel=0;
	(*Area)++;
	(*CenterX)+=c;
	(*CenterY)+=r;
	if (r > 0)
	  {
	  for (i=0; i<BPP; i++)
	    if (InputImage[((r-1)*COLS+c)*BPP+i] != InputImage[(y*COLS+x)*BPP+i])
		  break;
	  if (i < BPP)
		border_pixel=1;
	  }
	if (r < ROWS-1)
	  {
	  for (i=0; i<BPP; i++)
	    if (InputImage[((r+1)*COLS+c)*BPP+i] != InputImage[(y*COLS+x)*BPP+i])
		  break;
	  if (i < BPP)
		border_pixel=1;
	  }
	if (c > 0)
	  {
	  for (i=0; i<BPP; i++)
	    if (InputImage[(r*COLS+c-1)*BPP+i] != InputImage[(y*COLS+x)*BPP+i])
		  break;
	  if (i < BPP)
		border_pixel=1;
	  }
	if (c < COLS-1)
	  {
	  for (i=0; i<BPP; i++)
	    if (InputImage[(r*COLS+c+1)*BPP+i] != InputImage[(y*COLS+x)*BPP+i])
		  break;
	  if (i < BPP)
		border_pixel=1;
	  }
	if (border_pixel == 1)
	  border++;
	}
(*CenterX)/=(*Area);
(*CenterY)/=(*Area);
(*Compactness)=(double)(border*border)/(double)(*Area);
}



unsigned char	*RawImage,*ThreshImage;	/* for tieing ThresholdImage to ThresholdProc */
int				Width,Height,cumhist[256];

void ThresholdImage(HINSTANCE hInst,
					HWND hParentWnd,
					unsigned char *InputImage,
					unsigned char *OutputImage,
					int tROWS,int tCOLS)

{
int	  i;

RawImage=InputImage;
ThreshImage=OutputImage;
Width=tCOLS; Height=tROWS;
for (i=0; i<256; i++)
  cumhist[i]=0;
for (i=0; i<tROWS*tCOLS; i++)
  cumhist[InputImage[i]]++;
for (i=254; i>=0; i--)
  cumhist[i]+=cumhist[i+1];
DialogBox(hInst,"ID_THRESH_DIALOG",hParentWnd,(DLGPROC)ThresholdProc);
}



LRESULT CALLBACK ThresholdProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
static int	Threshold,Count;
SCROLLINFO	BarInfo;
BOOL		Success;
HWND		ScrollHwnd;
int			i;

switch(uMsg)
  {
  case WM_INITDIALOG:
	Threshold=128;
	SetDlgItemInt(hDlg,ID_EDIT_THRESH,Threshold,FALSE);
	Count=cumhist[Threshold];
	SetDlgItemInt(hDlg,ID_EDIT_PIXELCOUNT,Count,FALSE);
	BarInfo.cbSize=sizeof(SCROLLINFO);
	BarInfo.nMin=0;
	BarInfo.nMax=255;
	BarInfo.nPos=Threshold;
	BarInfo.fMask=SIF_POS | SIF_RANGE;
	ScrollHwnd=GetDlgItem(hDlg,ID_SCROLL_THRESH);
	SetScrollInfo(ScrollHwnd,SB_CTL,&BarInfo,TRUE);
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case ID_EDIT_THRESH:
		Threshold=GetDlgItemInt(hDlg,ID_EDIT_THRESH,&Success,FALSE);
		if (Threshold < 0) Threshold=0;
		if (Threshold > 255) Threshold=255;
		Count=cumhist[Threshold];
		break;
	  case ID_EDIT_PIXELCOUNT:
		break;	  // causes problems
		Count=GetDlgItemInt(hDlg,ID_EDIT_PIXELCOUNT,&Success,FALSE);
		if (Count < 0) Count=0;
		if (Count > Width*Height) Count=Width*Height;
		break;
	  case IDOK:
		Threshold=GetDlgItemInt(hDlg,ID_EDIT_THRESH,&Success,FALSE);
		for (i=0; i<Width*Height; i++)
		  if (RawImage[i] >= Threshold)
			ThreshImage[i]=255;
		  else
			ThreshImage[i]=0;
		EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  case WM_HSCROLL:
	if (LOWORD(wParam) == SB_LINELEFT) Threshold--;
	if (LOWORD(wParam) == SB_LINERIGHT) Threshold++;
	if (LOWORD(wParam) == SB_PAGELEFT) Threshold-=10;
	if (LOWORD(wParam) == SB_PAGERIGHT) Threshold+=10;
	if (LOWORD(wParam) == SB_THUMBPOSITION) Threshold=HIWORD(wParam);
	if (Threshold < 0) Threshold=0;
	if (Threshold > 255) Threshold=255;
	BarInfo.cbSize=sizeof(SCROLLINFO);
	BarInfo.fMask=SIF_POS;
	BarInfo.nPos=Threshold;
	ScrollHwnd=GetDlgItem(hDlg,ID_SCROLL_THRESH);
	SetScrollInfo(ScrollHwnd,SB_CTL,&BarInfo,TRUE);
	SetDlgItemInt(hDlg,ID_EDIT_THRESH,Threshold,FALSE);
	Count=cumhist[Threshold];
	SetDlgItemInt(hDlg,ID_EDIT_PIXELCOUNT,Count,FALSE);
	for (i=0; i<Width*Height; i++)
	  if (RawImage[i] >= Threshold)
		ThreshImage[i]=255;
	  else
		ThreshImage[i]=0;
	MakeDisplayImage(ThreshImage,Height,Width,1);
	PaintImage();
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}



/*
**	This routine applies histogram equalization to each band individually.
*/

void HistogramEqualize(unsigned char *InputImage,
					 unsigned char *EqImage,
					 int nrows,int ncols,int nbytes)

{
int		hist[256],i,b;
double	NormalizedHistogram[256],CumDistFunct[256];

for (b=0; b<nbytes; b++)
  {
  for (i=0; i<256; i++)
	hist[i]=0;
  for (i=0; i<nrows*ncols; i++)
	hist[InputImage[i*nbytes+b]]++;

  for (i=0; i<256; i++)	/* ignore white/black (hist[0] and hist[255]) */
    NormalizedHistogram[i]=(double)hist[i]/((double)(nrows*ncols-hist[255]-hist[0]));
  CumDistFunct[1]=NormalizedHistogram[1];
  for (i=2; i<255; i++)
    CumDistFunct[i]=CumDistFunct[i-1]+NormalizedHistogram[i];

  for (i=0; i<nrows*ncols; i++)
	if (InputImage[i*nbytes+b] == 0)
	  EqImage[i*nbytes+b]=0;
	else if (InputImage[i*nbytes+b] == 255)
	  EqImage[i*nbytes+b]=255;
	else
      EqImage[i*nbytes+b]=(unsigned char)((double)255.0*CumDistFunct[InputImage[i*nbytes+b]]);
  }
}


