#include <stdio.h>
#include <windows.h>
#include <winuser.h>
#include "resource.h"
#include "globals.h"

#define MAX_POLYS		10
#define MAX_POLY_POINTS	20
#define POLYPOINT_DELTA	3

static int				DISPLAY_ROWS,DISPLAY_COLS;	// display window
static unsigned char	*disp_image;				// display data
int						PolyPointsX[MAX_POLYS][MAX_POLY_POINTS]; // for clipping, etc.
int						PolyPointsY[MAX_POLYS][MAX_POLY_POINTS];
int						TotalPoints[MAX_POLYS],TotalPolys;


void DisplayImagePointer(unsigned char **ImagePointer,
						 int *ImageRows,int *ImageCols)

{
int	  i;

(*ImagePointer)=(unsigned char *)calloc(DISPLAY_ROWS*DISPLAY_COLS*3,1);
for (i=0; i<DISPLAY_ROWS*DISPLAY_COLS; i++)
  {
  (*ImagePointer)[i*3+0]=disp_image[i*3+2];
  (*ImagePointer)[i*3+1]=disp_image[i*3+1];
  (*ImagePointer)[i*3+2]=disp_image[i*3+0];
  }
*ImageRows=DISPLAY_ROWS;
*ImageCols=DISPLAY_COLS;
}


void MakeDisplayImage(unsigned char *Image,
					  int IMAGE_ROWS,int IMAGE_COLS,
					  int BYTES_PER_PIXEL)

{
int				r,c,r2,c2,i,rCell,cCell;
static int		LastRows=0,LastCols=0;
SCROLLINFO		BarPos;
RECT			MainRect;


GetClientRect(MainWnd,&MainRect);
DISPLAY_ROWS=MainRect.bottom-MainRect.top;
DISPLAY_COLS=MainRect.right-MainRect.left;
if (DISPLAY_ROWS < 100)
  DISPLAY_ROWS=100;
if (DISPLAY_COLS < 100)
  DISPLAY_COLS=100;
if (DISPLAY_ROWS % 4 != 0)	// Windows pads to 4-byte boundaries
  DISPLAY_ROWS=(DISPLAY_ROWS/4+1)*4;
if (DISPLAY_COLS % 4 != 0)
  DISPLAY_COLS=(DISPLAY_COLS/4+1)*4;

if (ScaleImageToWindow == 0  &&  DISPLAY_ROWS < IMAGE_ROWS*MagFact)
  {
  if (TopDispRow >= IMAGE_ROWS-DISPLAY_ROWS/MagFact)
	TopDispRow=IMAGE_ROWS-DISPLAY_ROWS/MagFact-1;
  if (TopDispRow < 0) TopDispRow=0;
  BarPos.cbSize=sizeof(SCROLLINFO);
  BarPos.fMask=SIF_ALL;
  BarPos.nMin=0; BarPos.nMax=IMAGE_ROWS;
  BarPos.nPage=DISPLAY_ROWS/MagFact;
  BarPos.nPos=TopDispRow;
  SetScrollInfo(MainWnd,SB_VERT,&BarPos,TRUE);
  ShowScrollBar(MainWnd,SB_VERT,TRUE);
  }
else
  {
  if (ScaleImageToWindow == 0)
	TopDispRow=0;	// entire height fits in window
  ShowScrollBar(MainWnd,SB_VERT,FALSE);
  }
if (ScaleImageToWindow == 0  &&  DISPLAY_COLS < IMAGE_COLS*MagFact)
  {
  if (LeftDispCol >= IMAGE_COLS-DISPLAY_COLS/MagFact)
	LeftDispCol=IMAGE_COLS-DISPLAY_COLS/MagFact-1;
  if (LeftDispCol < 0) LeftDispCol=0;
  BarPos.cbSize=sizeof(SCROLLINFO);
  BarPos.fMask=SIF_ALL;
  BarPos.nMin=0; BarPos.nMax=IMAGE_COLS;
  BarPos.nPage=DISPLAY_COLS/MagFact;
  BarPos.nPos=LeftDispCol;
  SetScrollInfo(MainWnd,SB_HORZ,&BarPos,TRUE);
  ShowScrollBar(MainWnd,SB_HORZ,TRUE);
  }
else
  {
  if (ScaleImageToWindow == 0)
	LeftDispCol=0;	// entire width fits in window
  ShowScrollBar(MainWnd,SB_HORZ,FALSE);
  }

if (disp_image != NULL  &&  (LastRows != DISPLAY_ROWS  ||  LastCols != DISPLAY_COLS))
  {
  free(disp_image);
  disp_image=NULL;
  }
if (disp_image == NULL)
  {
  disp_image=(unsigned char *)calloc(DISPLAY_ROWS*DISPLAY_COLS*3,1);
  if (disp_image == NULL)
    {
    MessageBox(MainWnd,"Out of memory","MakeDisplayImage()",MB_APPLMODAL | MB_OK);
    exit(0);
    }
  }


if (Image != NULL  &&  IMAGE_ROWS > 0  &&  IMAGE_COLS > 0)
  {			// have some data to display (not a blank image)
  if (ScaleImageToWindow == 0)
	{		// image will be displayed by mag factor
	if (DisplayLabels == 1  &&  LabelImage != NULL  &&
		IMAGE_ROWS == ROWS  &&  IMAGE_COLS == COLS)
	  {		// also displaying labels (so long as image is same size)
	  r2=TopDispRow;
	  c2=LeftDispCol;
	  r=0; c=0; rCell=0; cCell=0;
	  while (r < DISPLAY_ROWS)
	    {
	    while (c < DISPLAY_COLS)
	      {
	      if (r2 < 0  ||  r2 >= IMAGE_ROWS  ||
			  c2 < 0  ||  c2 >= IMAGE_COLS)
	        {
		    disp_image[(r*DISPLAY_COLS+c)*3+0]=0;
		    disp_image[(r*DISPLAY_COLS+c)*3+1]=0;
		    disp_image[(r*DISPLAY_COLS+c)*3+2]=0;
		    }
	      else if (rCell < MagFact/4  ||  rCell >= MagFact*3/4  ||
					cCell < MagFact/4  ||  cCell >= MagFact*3/4  ||
					LabelImage[r2*IMAGE_COLS+c2] == 0)
	        {
			i=(r2*IMAGE_COLS+c2)*BYTES_PER_PIXEL;
			disp_image[(r*DISPLAY_COLS+c)*3+2]=Image[i+0];
			disp_image[(r*DISPLAY_COLS+c)*3+1]=Image[i+BYTES_PER_PIXEL/3];
			disp_image[(r*DISPLAY_COLS+c)*3+0]=Image[i+BYTES_PER_PIXEL/3*2];
			}
		  else
			{
			i=LabelImage[r2*IMAGE_COLS+c2];
			disp_image[(r*DISPLAY_COLS+c)*3+2]=LabelColor[i][0];
			disp_image[(r*DISPLAY_COLS+c)*3+1]=LabelColor[i][1];
			disp_image[(r*DISPLAY_COLS+c)*3+0]=LabelColor[i][2];
			}
	      c++;
	      cCell++;
	      if (cCell == MagFact)
			{ cCell=0; c2++; }
	      }
		r++; c=0;
		rCell++; cCell=0; c2=LeftDispCol;
		if (rCell == MagFact)
	      { rCell=0; r2++; }
		}

	  }
	else	// not displaying labels, but still using mag factor
	  {
	  r2=TopDispRow;
	  c2=LeftDispCol;
	  r=0; c=0; rCell=0; cCell=0;
	  while (r < DISPLAY_ROWS)
	    {
	    while (c < DISPLAY_COLS)
	      {
	      if (r2 < 0  ||  r2 >= IMAGE_ROWS  ||
			  c2 < 0  ||  c2 >= IMAGE_COLS)
	        {
		    disp_image[(r*DISPLAY_COLS+c)*3+0]=0;
		    disp_image[(r*DISPLAY_COLS+c)*3+1]=0;
		    disp_image[(r*DISPLAY_COLS+c)*3+2]=0;
		    }
	      else
	        {
			i=(r2*IMAGE_COLS+c2)*BYTES_PER_PIXEL;
			disp_image[(r*DISPLAY_COLS+c)*3+2]=Image[i+0];
			disp_image[(r*DISPLAY_COLS+c)*3+1]=Image[i+BYTES_PER_PIXEL/3];
			disp_image[(r*DISPLAY_COLS+c)*3+0]=Image[i+BYTES_PER_PIXEL/3*2];
			}
	      c++;
	      cCell++;
	      if (cCell == MagFact)
			{ cCell=0; c2++; }
	      }
		r++; c=0;
		rCell++; cCell=0; c2=LeftDispCol;
		if (rCell == MagFact)
	      { rCell=0; r2++; }
		}
	  }
	}
  else		// fitting image data to window; never display labels
    {
	for (r=0; r<DISPLAY_ROWS; r++)
      {
      r2=(int)((double)r/(double)DISPLAY_ROWS*(double)IMAGE_ROWS);
      for (c=0; c<DISPLAY_COLS; c++)
	    {
	    c2=(int)((double)c/(double)DISPLAY_COLS*(double)IMAGE_COLS);
		i=(r2*IMAGE_COLS+c2)*BYTES_PER_PIXEL;
        disp_image[(r*DISPLAY_COLS+c)*3+2]=Image[i+0];
        disp_image[(r*DISPLAY_COLS+c)*3+1]=Image[i+BYTES_PER_PIXEL/3];
        disp_image[(r*DISPLAY_COLS+c)*3+0]=Image[i+BYTES_PER_PIXEL/3*2];
		}
	  }
	}
  }
else		// no image data given; create blank image
  for (r=0; r<DISPLAY_ROWS*DISPLAY_COLS*3; r++)
	disp_image[r]=0;
LastRows=DISPLAY_ROWS;
LastCols=DISPLAY_COLS;
}



void PaintImage()

{
PAINTSTRUCT			Painter;
HDC					hDC;
BITMAPINFOHEADER	bm_info_header;
BITMAPINFO			bm_info;

BeginPaint(MainWnd,&Painter);
hDC=GetDC(MainWnd);
bm_info_header.biSize=sizeof(BITMAPINFOHEADER); 
bm_info_header.biWidth=DISPLAY_COLS;
bm_info_header.biHeight=-DISPLAY_ROWS; 
bm_info_header.biPlanes=1;
bm_info_header.biBitCount=24; 
bm_info_header.biCompression=BI_RGB; 
bm_info_header.biSizeImage=0; 
bm_info_header.biXPelsPerMeter=0; 
bm_info_header.biYPelsPerMeter=0;
bm_info_header.biClrUsed=0; 
bm_info_header.biClrImportant=0;
// bm_info.bmiColors=NULL;
bm_info.bmiHeader=bm_info_header;
SetDIBitsToDevice(hDC,0,0,DISPLAY_COLS,DISPLAY_ROWS,0,0,
			  0, /* first scan line */
			  DISPLAY_ROWS, /* number of scan lines */
			  disp_image,&bm_info,DIB_RGB_COLORS);
ReleaseDC(MainWnd,hDC);
EndPaint(MainWnd,&Painter);
}



		/* returns the value (RGB) of the given pixel under display,
		** and also returns the row,col coordinate in the original image */

void GetImageValue(int *xPos,int *yPos,int nrows,int ncols,
				   int *red,int *green,int *blue)

{
int		x,y;

if (*xPos >= 0  &&  *xPos < DISPLAY_COLS  &&  *yPos >= 0  &&  *yPos < DISPLAY_ROWS)
  {
  if (ScaleImageToWindow == 0)
    {
	y=(*yPos)-((*yPos)%MagFact);
	x=(*xPos)-((*xPos)%MagFact);
    *red=(int)disp_image[(y*DISPLAY_COLS+x)*3+2];
    *green=(int)disp_image[(y*DISPLAY_COLS+x)*3+1];
    *blue=(int)disp_image[(y*DISPLAY_COLS+x)*3+0];
    }
  else
    {
    *red=(int)disp_image[((*yPos)*DISPLAY_COLS+(*xPos))*3+2];
    *green=(int)disp_image[((*yPos)*DISPLAY_COLS+(*xPos))*3+1];
    *blue=(int)disp_image[((*yPos)*DISPLAY_COLS+(*xPos))*3+0];
    }
  }
if (ScaleImageToWindow == 1)
  {
  *xPos=(int)((double)(*xPos)/(double)DISPLAY_COLS*(double)ncols);
  *yPos=(int)((double)(*yPos)/(double)DISPLAY_ROWS*(double)nrows);
  }
else
  {
  *xPos=LeftDispCol+((*xPos)/MagFact);
  *yPos=TopDispRow+((*yPos)/MagFact);
  }
}


		/* draw a point on the image the given color; -1 means draw display image color */
		/* MarkerSize is how big to draw it; -1 means use MagFact for scale */

void DrawPixelColor(int x,int y,int ImageRows,int ImageCols,int Color,int MarkerSize)

{
HDC			hDC;
HBRUSH		DrawBrush;
RECT		LabelRect;
int			xDisp,yDisp;


if (ScaleImageToWindow == 1)
  {
  yDisp=(int)((double)y*(double)DISPLAY_ROWS/(double)ImageRows);
  xDisp=(int)((double)x*(double)DISPLAY_COLS/(double)ImageCols);
  if (MarkerSize == -1)
    MarkerSize=2;
  LabelRect.left=xDisp-MarkerSize/2;
  LabelRect.right=xDisp+MarkerSize/2;
  LabelRect.top=yDisp-MarkerSize/2;
  LabelRect.bottom=yDisp+MarkerSize/2;
  }
else
  {
  yDisp=(y-TopDispRow)*MagFact;
  xDisp=(x-LeftDispCol)*MagFact;
  if (MarkerSize == -1)
    MarkerSize=MagFact;
  if (xDisp < 0  ||  xDisp+MarkerSize >= DISPLAY_COLS  ||  yDisp < 0  ||  yDisp+MarkerSize >= DISPLAY_ROWS)
    return;	  /* pixel is not visible in window */
  LabelRect.left=xDisp+MarkerSize/4;
  LabelRect.right=xDisp+MarkerSize*3/4;
  LabelRect.top=yDisp+MarkerSize/4;
  LabelRect.bottom=yDisp+MarkerSize*3/4;
  }
hDC=GetDC(MainWnd);
if (Color == -1) /* undraw */
  DrawBrush=CreateSolidBrush(RGB(disp_image[(yDisp*DISPLAY_COLS+xDisp)*3+2],
								   disp_image[(yDisp*DISPLAY_COLS+xDisp)*3+1],
								   disp_image[(yDisp*DISPLAY_COLS+xDisp)*3+0]));
else
  DrawBrush=CreateSolidBrush(RGB(0,0,Color));
FillRect(hDC,&LabelRect,DrawBrush);
DeleteObject(DrawBrush);
ReleaseDC(MainWnd,hDC);
}


		/* draw a snakes contour on the image */
		/* DrawFlag = 1 means draw it, DrawFlag = 0 means undraw */

void DrawContourOnImage(int ImageRows,int ImageCols,int DrawFlag)

{
int			i;


for (i=0; i<TotalContourPoints; i++)
  {
  DrawPixelColor(ContourX[i],ContourY[i],ImageRows,ImageCols,(DrawFlag == 1 ? 255 : -1),8);
  }
}




void SetLabelImage(int xDisp,int yDisp,int xImg,int yImg,int SetLabel)

{
HDC			hDC;
HBRUSH		DrawBrush;
RECT		LabelRect;

if (xImg < 0  ||  xImg >= COLS  ||  yImg < 0  ||  yImg >= ROWS)
  return;
LabelImage[yImg*COLS+xImg]=SetLabel;
if (MagFact < 2)
  return;
LabelRect.left=xDisp-(xDisp%MagFact)+MagFact/4;
LabelRect.right=xDisp-(xDisp%MagFact)+MagFact*3/4;
LabelRect.top=yDisp-(yDisp%MagFact)+MagFact/4;
LabelRect.bottom=yDisp-(yDisp%MagFact)+MagFact*3/4;
hDC=GetDC(MainWnd);
if (SetLabel == 0) /* unlabel */
  DrawBrush=CreateSolidBrush(RGB(
		disp_image[(LabelRect.top*DISPLAY_COLS+LabelRect.left)*3+2],
		disp_image[(LabelRect.top*DISPLAY_COLS+LabelRect.left)*3+1],
		disp_image[(LabelRect.top*DISPLAY_COLS+LabelRect.left)*3+0]));
else
  DrawBrush=CreateSolidBrush(RGB(LabelColor[SetLabel][0],
			LabelColor[SetLabel][1],LabelColor[SetLabel][2]));
FillRect(hDC,&LabelRect,DrawBrush);
DeleteObject(DrawBrush);
ReleaseDC(MainWnd,hDC);
}


#define MAX_PAINT 1000

void PaintLabelImage(int xImg,int yImg,
					 int EraseFlag)		// 0 => paint, 1 => erase

{
static int	Indices[MAX_PAINT],PaintedOverLabel,Total;
int			i,r,c,Head;

if (xImg < 0  ||  xImg >= COLS  ||  yImg < 0  ||  yImg >= ROWS)
  return;
if (EraseFlag)
  {
  for (i=0; i<Total; i++)
	LabelImage[Indices[i]]=PaintedOverLabel;
  return;
  }
if (LabelImage[yImg*COLS+xImg] == Label)
  return;

PaintedOverLabel=LabelImage[yImg*COLS+xImg];
Indices[0]=yImg*COLS+xImg;
LabelImage[yImg*COLS+xImg]=Label;
Total=1;
Head=0;
while (Total < MAX_PAINT  &&  Head < Total)
  {
  r=Indices[Head]/COLS;
  c=Indices[Head]%COLS;
  if (r-1 > 0  &&  LabelImage[(r-1)*COLS+c] == PaintedOverLabel)
	{
	LabelImage[(r-1)*COLS+c]=Label;
	Indices[Total]=(r-1)*COLS+c;
	Total++;
	if (Total >= MAX_PAINT)
	  break;
	}
  if (c+1 < COLS  &&  LabelImage[r*COLS+c+1] == PaintedOverLabel)
	{
	LabelImage[r*COLS+c+1]=Label;
	Indices[Total]=r*COLS+c+1;
	Total++;
	if (Total >= MAX_PAINT)
	  break;
	}
  if (r+1 < ROWS  &&  LabelImage[(r+1)*COLS+c] == PaintedOverLabel)
	{
	LabelImage[(r+1)*COLS+c]=Label;
	Indices[Total]=(r+1)*COLS+c;
	Total++;
	if (Total >= MAX_PAINT)
	  break;
	}
  if (c-1 < COLS  &&  LabelImage[r*COLS+c-1] == PaintedOverLabel)
	{
	LabelImage[r*COLS+c-1]=Label;
	Indices[Total]=r*COLS+c-1;
	Total++;
	}
  Head++;
  }
}



void GetPolyLine(HWND DrawWnd,
				  int *x1,int *y1,int *x2,int *y2)

{
MSG					msg;
POINT				ptCursor;
int					DrawingPoly;


TotalPolys=TotalPoints[0]=0;
DrawingPoly=0;
*x1=*y1=*x2=*y2=0;
while (1)
  {
  GetMessage(&msg,DrawWnd,0,0);
  if (msg.message == WM_LBUTTONDOWN)
    {
	GetCursorPos(&ptCursor);
	ScreenToClient(DrawWnd,&ptCursor);
	if (DrawingPoly == 0)
      {
	  *x1=ptCursor.x;
	  *y1=ptCursor.y;
	  DrawingPoly=1;
	  PolyPointsX[0][0]=ptCursor.x;
	  PolyPointsY[0][0]=ptCursor.y;
	  PolyPointsX[0][1]=ptCursor.x;
	  PolyPointsY[0][1]=ptCursor.y;
	  TotalPoints[0]=2;
	  TotalPolys=1;
	  }
	else
      {
	  *x2=ptCursor.x;
	  *y2=ptCursor.y;
	  break;
	  }
    }
  else if (msg.message == WM_RBUTTONDOWN)
    {
	DrawAllPolygons(DrawWnd,0);
	DrawingPoly=0;
	TotalPoints[0]=0;
	TotalPolys=0;
    }
  else if (msg.message == WM_MOUSEMOVE)
	{
	GetCursorPos(&ptCursor);
	ScreenToClient(DrawWnd,&ptCursor);
	if (DrawingPoly)
	  {
	  DrawAllPolygons(DrawWnd,0);
	  PolyPointsX[0][1]=ptCursor.x;
	  PolyPointsY[0][1]=ptCursor.y;
	  DrawAllPolygons(DrawWnd,1);
	  }
    }
  else if (msg.message == WM_PAINT)
    {
	PaintImage();
	DrawAllPolygons(DrawWnd,1);
    }
  else
    DefWindowProc(DrawWnd,msg.message,msg.wParam,msg.lParam);
  }
}



void GetPolyPoint(HWND DrawWnd,
				  int *x1,int *y1)

{
MSG					msg;
POINT				ptCursor;

*x1=*y1=0;
while (1)
  {
  GetMessage(&msg,DrawWnd,0,0);
  if (msg.message == WM_LBUTTONDOWN)
    {
	GetCursorPos(&ptCursor);
	ScreenToClient(DrawWnd,&ptCursor);
	*x1=ptCursor.x;
	*y1=ptCursor.y;
	break;
    }
  else
    DefWindowProc(DrawWnd,msg.message,msg.wParam,msg.lParam);
  }
}




void GetPolyBox(HWND DrawWnd,
				  int *x1,int *y1,int *x2,int *y2)

{
MSG					msg;
POINT				ptCursor;
int					DrawingPoly;


TotalPolys=TotalPoints[0]=0;
DrawingPoly=0;
*x1=*y1=*x2=*y2=0;
while (1)
  {
  GetMessage(&msg,DrawWnd,0,0);
  if (msg.message == WM_LBUTTONDOWN)
    {
	GetCursorPos(&ptCursor);
	ScreenToClient(DrawWnd,&ptCursor);
	if (DrawingPoly == 0)
      {
	  *x1=ptCursor.x;
	  *y1=ptCursor.y;
	  DrawingPoly=1;
	  PolyPointsX[0][0]=ptCursor.x;
	  PolyPointsY[0][0]=ptCursor.y;
	  PolyPointsX[0][1]=ptCursor.x;
	  PolyPointsY[0][1]=ptCursor.y;
	  PolyPointsX[0][2]=ptCursor.x;
	  PolyPointsY[0][2]=ptCursor.y;
	  PolyPointsX[0][3]=ptCursor.x;
	  PolyPointsY[0][3]=ptCursor.y;
	  TotalPoints[0]=4;
	  TotalPolys=1;
	  }
	else
      {
	  *x2=ptCursor.x;
	  *y2=ptCursor.y;
	  break;
	  }
    }
  else if (msg.message == WM_RBUTTONDOWN)
    {
	DrawAllPolygons(DrawWnd,0);
	DrawingPoly=0;
	TotalPoints[0]=0;
	TotalPolys=0;
    }
  else if (msg.message == WM_MOUSEMOVE)
	{
	GetCursorPos(&ptCursor);
	ScreenToClient(DrawWnd,&ptCursor);
	if (DrawingPoly)
	  {
	  DrawAllPolygons(DrawWnd,0);
	  PolyPointsX[0][1]=ptCursor.x;
	  PolyPointsY[0][1]=PolyPointsY[0][0];
	  PolyPointsX[0][2]=ptCursor.x;
	  PolyPointsY[0][2]=ptCursor.y;
	  PolyPointsX[0][3]=PolyPointsX[0][0];
	  PolyPointsY[0][3]=ptCursor.y;
	  DrawAllPolygons(DrawWnd,1);
	  }
    }
  else if (msg.message == WM_PAINT)
    {
	PaintImage();
	DrawAllPolygons(DrawWnd,1);
    }
  else
    DefWindowProc(DrawWnd,msg.message,msg.wParam,msg.lParam);
  }
}



void DrawAllPolygons(HWND DrawWnd,
					 int PaintFlag)

{
HDC		hDC;
int		i,p,l,x,y,x1,x2,y1,y2,indices[1000],tp,color;

hDC=GetDC(DrawWnd);
for (p=0; p<TotalPolys; p++)
  for (l=0; l<TotalPoints[p]; l++)
	{
	x1=PolyPointsX[p][l];
	y1=PolyPointsY[p][l];
	x2=PolyPointsX[p][(l+1)%TotalPoints[p]];
	y2=PolyPointsY[p][(l+1)%TotalPoints[p]];
	MakeLineIndices(x1,y1,x2,y2,DISPLAY_ROWS,DISPLAY_COLS,indices,&tp);
    color=(255 << 16)+(100 << 8)+100;
	for (i=0; i<tp; i++)
	  {
	  if (PaintFlag == 0)
		color=(disp_image[indices[i]*3+0] << 16)+
				(disp_image[indices[i]*3+1] << 8)+
				(disp_image[indices[i]*3+2]);
	  SetPixel(hDC,indices[i]%DISPLAY_COLS,indices[i]/DISPLAY_COLS,color);
	  }
	for (x=x1-POLYPOINT_DELTA; x<=x1+POLYPOINT_DELTA; x++)
	  for (y=y1-POLYPOINT_DELTA; y<=y1+POLYPOINT_DELTA; y++)
		{
	    if (PaintFlag == 0)
		  color=(disp_image[(y*DISPLAY_COLS+x)*3+0] << 16)+
				(disp_image[(y*DISPLAY_COLS+x)*3+1] << 8)+
				(disp_image[(y*DISPLAY_COLS+x)*3+2]);
		SetPixel(hDC,x,y,color);
		}
    }
ReleaseDC(DrawWnd,hDC);
}




static char		ReturnFilename[MAX_FILENAME_CHARS];
static int		ReturnImageType,UserCanSelectType;

BOOL SetFilename(HINSTANCE hInst,
				 HWND hParentWnd,
				 char *Filename,	/* returned name of file selected */
				 int *ImageType)	/* returned type of file selected */
									/* if input is negative, user is not */
									/* allowed to change file type */

{
char	*FileOnly;

if (*ImageType < 0)
  { UserCanSelectType=0; ReturnImageType=0-(*ImageType); }
else
  { UserCanSelectType=1; ReturnImageType=*ImageType; }
DialogBox(hInst,"ID_FILE_MENU",hParentWnd,(DLGPROC)OpenFileProc);
if (strlen(ReturnFilename) > 1)
  {		/* set current path to whatever was used to select filename */
  GetFullPathName(ReturnFilename,MAX_FILENAME_CHARS,CurrentPath,&FileOnly);
  *FileOnly='\0';
		/* copy return filename to calling filename */
  strcpy(Filename,ReturnFilename);
  *ImageType=ReturnImageType;
  return(TRUE);
  }
return(FALSE);
}



LRESULT CALLBACK OpenFileProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
char			*FileOnly,CurrentList[MAX_FILENAME_CHARS];
static char		Suffix[5];

switch(uMsg)
  {
  case WM_INITDIALOG:
	if (UserCanSelectType == 0)
	  {
	  if (ReturnImageType == 2)
		{
	    strcpy(Suffix,"JPG");
	    CheckRadioButton(hDlg,IDD_JPEG,IDD_PPM,IDD_JPEG);
		}
	  else	// default
		{
	    strcpy(Suffix,"P?M");
	    CheckRadioButton(hDlg,IDD_JPEG,IDD_PPM,IDD_PPM);
		}
	  }
	else	// default 
	  {
	  strcpy(Suffix,"P?M");
	  CheckRadioButton(hDlg,IDD_JPEG,IDD_PPM,IDD_PPM);
	  ReturnImageType=1;
	  }
	sprintf(ReturnFilename,"%s*.%s",CurrentPath,Suffix);
    DlgDirList(hDlg,ReturnFilename,IDC_LIST,IDC_DIRECTORY,DDL_DIRECTORY | DDL_DRIVES);
	GetFullPathName(ReturnFilename,MAX_FILENAME_CHARS,CurrentList,&FileOnly);
	SetDlgItemText(hDlg,IDC_DIRECTORY,CurrentList);
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case IDD_JPEG:
		if (UserCanSelectType == 0)
		  break;
		strcpy(Suffix,"JPG");
		ReturnFilename[strlen(ReturnFilename)-3]='\0';
		lstrcat(ReturnFilename,Suffix);
	    DlgDirList(hDlg,ReturnFilename,IDC_LIST,IDC_DIRECTORY,DDL_DIRECTORY | DDL_DRIVES);
		GetFullPathName(ReturnFilename,MAX_FILENAME_CHARS,CurrentList,&FileOnly);
		SetDlgItemText(hDlg,IDC_DIRECTORY,CurrentList);
		CheckRadioButton(hDlg,IDD_JPEG,IDD_PPM,IDD_JPEG);
	    ReturnImageType=2;
		break;
	  case IDD_PPM:
		if (UserCanSelectType == 0)
		  break;
		strcpy(Suffix,"P?M");
		ReturnFilename[strlen(ReturnFilename)-3]='\0';
		lstrcat(ReturnFilename,Suffix);
	    DlgDirList(hDlg,ReturnFilename,IDC_LIST,IDC_DIRECTORY,DDL_DIRECTORY | DDL_DRIVES);
		GetFullPathName(ReturnFilename,MAX_FILENAME_CHARS,CurrentList,&FileOnly);
		SetDlgItemText(hDlg,IDC_DIRECTORY,CurrentList);
		CheckRadioButton(hDlg,IDD_JPEG,IDD_PPM,IDD_PPM);
	    ReturnImageType=1;
		break;
	  case IDC_LIST:
        if (HIWORD(wParam) == LBN_DBLCLK)
		  {
		  if (DlgDirSelectEx(hDlg,ReturnFilename,sizeof(ReturnFilename),IDC_LIST))
			{
			if (strlen(ReturnFilename) == 2  &&  ReturnFilename[1] == ':')
			  lstrcat(ReturnFilename,"\\");
			lstrcat(ReturnFilename,"*.");
			lstrcat(ReturnFilename,Suffix);
			DlgDirList(hDlg,ReturnFilename,IDC_LIST,IDC_DIRECTORY,DDL_DIRECTORY | DDL_DRIVES);
			GetFullPathName(ReturnFilename,MAX_FILENAME_CHARS,CurrentList,&FileOnly);
			SetDlgItemText(hDlg,IDC_DIRECTORY,CurrentList);
		    }
		  else
			{
			EndDialog(hDlg,IDOK);
		    }
		  }
		break;
	  case IDCANCEL:
		strcpy(ReturnFilename,"");
		EndDialog(hDlg,IDCANCEL);
		break;
	  case IDOK:
		GetDlgItemText(hDlg,IDC_DIRECTORY,ReturnFilename,MAX_FILENAME_CHARS);
		if (strlen(ReturnFilename) > 5  &&
			ReturnFilename[strlen(ReturnFilename)-5] == '*')
		  {		/* wildcards in edit box */
		  strcpy(ReturnFilename,"none?");
		  if (DlgDirSelectEx(hDlg,ReturnFilename,sizeof(ReturnFilename),IDC_LIST))
			{	/* user selected directory/drive */
		    MessageBox(hDlg,"Cannot select a directory\n(Try double-clicking.)",
					"Error",MB_OK | MB_APPLMODAL);
			}
		  else if (strcmp(ReturnFilename,"none?") == 0)
			{	/* nothing selected; user clicked ok */
		    strcpy(ReturnFilename,"");
		    EndDialog(hDlg,IDCANCEL);
		    }
		  else	/* something selected, user clicked ok */
			EndDialog(hDlg,IDOK);
		  break;
		  }
		else	/* user typed something in edit box */
		  EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}




LRESULT CALLBACK MagFactProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
SCROLLINFO	BarInfo;
BOOL		Success;
static HWND	ScrollHwnd,EditHwnd,ButtonHwnd;

switch(uMsg)
  {
  case WM_INITDIALOG:
	SetDlgItemInt(hDlg,ID_EDIT_THRESH,MagFact,FALSE);
	BarInfo.cbSize=sizeof(SCROLLINFO);
	BarInfo.nMin=1;
	BarInfo.nMax=64;
	BarInfo.nPos=MagFact;
	BarInfo.fMask=SIF_POS | SIF_RANGE;
	ScrollHwnd=GetDlgItem(hDlg,ID_SCROLL_THRESH);
	EditHwnd=GetDlgItem(hDlg,ID_EDIT_THRESH);
	ButtonHwnd=GetDlgItem(hDlg,IDD_FIT);
	SetScrollInfo(ScrollHwnd,SB_CTL,&BarInfo,TRUE);
	if (ScaleImageToWindow)
	  {
	  EnableWindow(ScrollHwnd,FALSE);
	  EnableWindow(EditHwnd,FALSE);
	  CheckDlgButton(hDlg,IDD_FIT,BST_CHECKED);
	  }
	else
	  {
	  EnableWindow(ScrollHwnd,TRUE);
	  EnableWindow(EditHwnd,TRUE);
	  CheckDlgButton(hDlg,IDD_FIT,BST_UNCHECKED);
	  }
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case ID_EDIT_THRESH:
		MagFact=GetDlgItemInt(hDlg,ID_EDIT_THRESH,&Success,FALSE);
		if (MagFact < 1) MagFact=1;
		if (MagFact > 64) MagFact=64;
		break;
	  case IDD_FIT:
		if (ScaleImageToWindow)
	      {
	      EnableWindow(ScrollHwnd,TRUE);
	      EnableWindow(EditHwnd,TRUE);
	      CheckDlgButton(hDlg,IDD_FIT,BST_UNCHECKED);
		  ScaleImageToWindow=0;
	      }
	    else
	      {
	      EnableWindow(ScrollHwnd,FALSE);
	      EnableWindow(EditHwnd,FALSE);
	      CheckDlgButton(hDlg,IDD_FIT,BST_CHECKED);
		  ScaleImageToWindow=1;
	      }
		break;
	  case IDOK:
		MagFact=GetDlgItemInt(hDlg,ID_EDIT_THRESH,&Success,FALSE);
		EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  case WM_HSCROLL:
	if (LOWORD(wParam) == SB_LINELEFT) MagFact--;
	if (LOWORD(wParam) == SB_LINERIGHT) MagFact++;
	if (LOWORD(wParam) == SB_PAGELEFT) MagFact/=2;
	if (LOWORD(wParam) == SB_PAGERIGHT) MagFact*=2;
	if (LOWORD(wParam) == SB_THUMBPOSITION) MagFact=HIWORD(wParam);
	if (MagFact < 1) MagFact=1;
	if (MagFact > 64) MagFact=64;
	BarInfo.cbSize=sizeof(SCROLLINFO);
	BarInfo.fMask=SIF_POS;
	BarInfo.nPos=MagFact;
	ScrollHwnd=GetDlgItem(hDlg,ID_SCROLL_THRESH);
	SetScrollInfo(ScrollHwnd,SB_CTL,&BarInfo,TRUE);
	SetDlgItemInt(hDlg,ID_EDIT_THRESH,MagFact,FALSE);
//	MakeDisplayImage(OriginalImage,ROWS,COLS,OriginalBPP);
//	PaintImage();
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}





LRESULT CALLBACK ProbeParamsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
SCROLLINFO	BarInfo;
BOOL		Success;
HWND		ScrollHwnd;

switch(uMsg)
  {
  case WM_INITDIALOG:
	SetDlgItemInt(hDlg,ID_EDIT_PARAMETERS,VesselsParameters,FALSE);
	BarInfo.cbSize=sizeof(SCROLLINFO);
	BarInfo.nMin=0;
	BarInfo.nMax=9;
	BarInfo.nPos=VesselsParameters;
	BarInfo.fMask=SIF_POS | SIF_RANGE;
	ScrollHwnd=GetDlgItem(hDlg,ID_SCROLL_PROBE);
	SetScrollInfo(ScrollHwnd,SB_CTL,&BarInfo,TRUE);
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case ID_EDIT_PARAMETERS:
		VesselsParameters=GetDlgItemInt(hDlg,ID_EDIT_PARAMETERS,&Success,FALSE);
		if (VesselsParameters < 0) VesselsParameters=0;
		if (VesselsParameters > 9) VesselsParameters=9;
		break;
	  case IDOK:
		VesselsParameters=GetDlgItemInt(hDlg,ID_EDIT_PARAMETERS,&Success,FALSE);
		EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  case WM_HSCROLL:
	if (LOWORD(wParam) == SB_LINELEFT) VesselsParameters--;
	if (LOWORD(wParam) == SB_LINERIGHT) VesselsParameters++;
	if (LOWORD(wParam) == SB_PAGELEFT) VesselsParameters-=3;
	if (LOWORD(wParam) == SB_PAGERIGHT) VesselsParameters+=3;
	if (LOWORD(wParam) == SB_THUMBPOSITION) VesselsParameters=HIWORD(wParam);
	if (VesselsParameters < 0) VesselsParameters=0;
	if (VesselsParameters > 9) VesselsParameters=9;
	BarInfo.cbSize=sizeof(SCROLLINFO);
	BarInfo.fMask=SIF_POS;
	BarInfo.nPos=VesselsParameters;
	ScrollHwnd=GetDlgItem(hDlg,ID_SCROLL_PROBE);
	SetScrollInfo(ScrollHwnd,SB_CTL,&BarInfo,TRUE);
	SetDlgItemInt(hDlg,ID_EDIT_PARAMETERS,VesselsParameters,FALSE);
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}



